using System;
using System.ComponentModel;
using System.Drawing;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;

namespace NinjaTrader.Indicator
{
    /// <summary>
    /// </summary>
    [Description("SuperDuperTrend2 Indicator modified from TSSuperTrend (TradingStudies.com (Version 2.2)) by Integrity-Traders.com")]
    public class SuperDuperTrend2 : Indicator
    {
		#region Variables
		
        private int _length = 10; 
        private double _multiplier = 2.01; 
        private bool _showArrows;
		private bool showEntries = true;
        private bool _colorBars = true;
		
		private bool _cciOverrides; 
		private int _cCIPeriods = 20; 
		private DataSeries CCISeries;
		
        private BoolSeries _trend;
        private IDataSeries _avg;
        private double _offset;
        private MovingAverageTypeSDT2 _maType = MovingAverageTypeSDT2.VWMA; 
        private SuperTrendModeSDT2 _smode = SuperTrendModeSDT2.ATR;
        private int _smooth = 1;
        private Color _barColorUp = Color.Blue;
        private Color _barColorDown = Color.Red;
		private Color _barColorDisagreement = Color.Gold;
        private Color _tempColor;
        private Color _prevColor;
		
		private CCI_Overrides2 _overridesType = CCI_Overrides2.None;
		private bool 	showTrendCandles 	= true;
		private bool 	showTrendCrossDots 	= true;
		private bool 	CCItrendUp, TrendUpDn;
		private BoolSeries CCIUpDn;
		private double highMarker, lowMarker;
		bool RangeType;
		int probeLongBar = 1;
		int probeShortBar = 1;
		private int ticksFromTrendline = 1;
		private int barsTillReset = 8;
		private Color _probeColorUp = Color.Cyan;
        private Color _probeColorDown = Color.Magenta;
		
		#endregion
		
        protected override void Initialize()
        {
			CCISeries    = new DataSeries(this);
			
			Add(new Plot(new Pen(Color.White,2), PlotStyle.Dot, "BreakTrend"));//Plot0
			Add(new Plot(new Pen(Color.Green,1), PlotStyle.Hash, "UpTrend"));//Plot1
			Add(new Plot(new Pen(Color.Red,1), PlotStyle.Hash, "DownTrend"));//Plot2
            
            CalculateOnBarClose = false;
            Overlay = true;
			
            _trend = new BoolSeries(this);
			CCIUpDn = new BoolSeries(this);
			
			if (_smooth > 1)
                switch (_maType)
                {
                    case MovingAverageTypeSDT2.SMA:
                        _avg = SMA(Input, _smooth);
                        break;
                    case MovingAverageTypeSDT2.SMMA:
                        _avg = SMMA(Input, _smooth);
                        break;
                    case MovingAverageTypeSDT2.TMA:
                        _avg = TMA(Input, _smooth);
                        break;
                    case MovingAverageTypeSDT2.WMA:
                        _avg = WMA(Input, _smooth);
                        break;
                    case MovingAverageTypeSDT2.VWMA:
                        _avg = VWMA(Input, _smooth);
                        break;
                    case MovingAverageTypeSDT2.TEMA:
                        _avg = TEMA(Input, _smooth);
                        break;
                    case MovingAverageTypeSDT2.HMA:
                        _avg = HMA(Input, _smooth);
                        break;
                    case MovingAverageTypeSDT2.VMA:
                        _avg = VMA(Input, _smooth, _smooth);
                        break;
                    default:
                        _avg = EMA(Input, _smooth);
                        break;
                }
            	else
                	_avg = Input;
            
        }

        protected override void OnBarUpdate()
        {
			if(CurrentBar == 0)
			{
				switch (_overridesType)
				{
					case CCI_Overrides2.None:
						_cciOverrides = false;
						break;
					case CCI_Overrides2.Override:
						_cciOverrides = true;
						break;
				}
			}
			
            if (CurrentBar < 1)
            {
                _trend.Set(true);
                UpTrend.Set(Close[0]);
                DownTrend.Set(Close[0]);
				return;
            } 
			
			switch (_smode)
			{
				case SuperTrendModeSDT2.ATR:
					_offset = ATR(_length)[0] * Multiplier;
					break;
				case SuperTrendModeSDT2.Adaptive:
					_offset = ATR(_length)[0] * HomodyneDiscriminator(Input)[0] / 10;
					break;
				default:
					_offset = DTT(_length, Multiplier);
					break;
			}
			
			if(showEntries)
			{
				if(Bars.Period.Id == PeriodType.Range || Bars.Period.Id == PeriodType.Final1 || Bars.Period.Id == PeriodType.Final2)//Range
				{
					RangeType = true;
					highMarker = Low[0]+ (Bars.Period.Value*TickSize) + TickSize;
					lowMarker = High[0]- (Bars.Period.Value*TickSize) - TickSize;
				}
				else
				if(Bars.Period.Id == PeriodType.Custom9)//Point-O
				{
					RangeType = true;
					highMarker = Close[1]+ (Bars.Period.Value*TickSize) + TickSize;
					lowMarker = Close[1]- (Bars.Period.Value*TickSize) - TickSize;
				}
				else	
				if(Bars.Period.Id == PeriodType.Custom5 || Bars.Period.Id == PeriodType.Custom6)//Renkos
				{
					RangeType = true;
					if(Close[1] > Open[1])
					{
						highMarker = Close[1] + (Bars.Period.Value*TickSize) + TickSize;
						lowMarker = Close[1] - ((Bars.Period.Value*TickSize) * 2) - TickSize;
					}
					if(Close[1] < Open[1])
					{
						highMarker = Close[1] + ((Bars.Period.Value*TickSize) * 2) + TickSize;
						lowMarker = Close[1] - (Bars.Period.Value*TickSize) - TickSize;
					}
				}
				else
					RangeType = false;
			}
			
			
            _trend.Set(Close[0] > DownTrend[1] ? true : Close[0] < UpTrend[1] ? false : _trend[1]);

			if (_trend[0] && !_trend[1])
            { 	
                UpTrend.Set(_avg[0] - _offset); if(showTrendCrossDots) BreakTrend.Set(DownTrend[1]);
				if (Plots[0].PlotStyle == PlotStyle.Line) UpTrend.Set(1, DownTrend[1]);
				TrendUpDn = true; DownTrend.Reset(0);
                if (ShowArrows)
                    DrawTriangleUp(CurrentBar.ToString(), true, 0, UpTrend[0] - TickSize, _barColorUp);
				if(showEntries && RangeType && !_cciOverrides)
					DrawText("EntryText" + CurrentBar.ToString(), true, FormatPrice(highMarker), 0, UpTrend[0] - TickSize * 2, -30, Color.FromArgb(255, ChartControl.AxisColor), new Font("Arial", 9, GraphicsUnit.Point), StringAlignment.Center, Color.FromArgb(255, ChartControl.AxisColor), Color.Empty, 0);
			}
            else if (!_trend[0] && _trend[1])
            {	
                DownTrend.Set(_avg[0] + _offset); if(showTrendCrossDots) BreakTrend.Set(UpTrend[1]);
                if (Plots[1].PlotStyle == PlotStyle.Line) DownTrend.Set(1, UpTrend[1]);
				TrendUpDn = false; UpTrend.Reset(0);
               	if (ShowArrows)
                    DrawTriangleDown(CurrentBar.ToString(), true, 0, DownTrend[0] + TickSize, _barColorDown);
				if(showEntries && RangeType && !_cciOverrides)
					DrawText("EntryText" + CurrentBar.ToString(), true, FormatPrice(lowMarker), 0, DownTrend[0] + TickSize * 2, 34, Color.FromArgb(255, ChartControl.AxisColor), new Font("Arial", 9, GraphicsUnit.Point), StringAlignment.Center, Color.FromArgb(255, ChartControl.AxisColor), Color.Empty, 0);
            }
            else
            {	
                if (_trend[0])
				{
                    UpTrend.Set((_avg[0] - _offset) > UpTrend[1] ? (_avg[0] - _offset) : UpTrend[1]);
					TrendUpDn = true; DownTrend.Reset(0);
				}
                else if (!_trend[0])
				{
                    DownTrend.Set((_avg[0] + _offset) < DownTrend[1] ? (_avg[0] + _offset) : DownTrend[1]);
					TrendUpDn = false; UpTrend.Reset(0);
				}
                RemoveDrawObject(CurrentBar.ToString());
				RemoveDrawObject("EntryText" + CurrentBar.ToString());
            }

			
			if(_cciOverrides)
			{
				CCIUpDn.Set(CCI(Input, _cCIPeriods)[0] > 0 ? true : CCI(Input, _cCIPeriods)[0] < 0 ? false : CCIUpDn[1]);
				/*
				if((Trend[0] && CCIUpDn[0] && !CCIUpDn[1]) || (CCIUpDn[0] && Trend[0] && !Trend[1]))
				{
					//DrawArrowUp("EntryArrows" + CurrentBar.ToString(), true, 0, UpTrend[0] - TickSize * 2, _barColorUp);
					if(RangeType) 
					{
						DrawText("EntryText" + CurrentBar.ToString(), true, FormatPrice(highMarker), 0, UpTrend[0] - TickSize * 2, -30, Color.FromArgb(255, ChartControl.AxisColor), new Font("Arial", 9, GraphicsUnit.Point), StringAlignment.Center, Color.FromArgb(255, ChartControl.AxisColor), Color.Empty, 0);
						DrawArrowUp("EntryArrows" + CurrentBar.ToString(), true, 0, UpTrend[0] - TickSize * 2, _barColorUp);
					}
				}
				else
				if((!Trend[0] && !CCIUpDn[0] && CCIUpDn[1]) || (!CCIUpDn[0] && !Trend[0] && Trend[1]))
				{
					//DrawArrowDown("EntryArrows" + CurrentBar.ToString(), true, 0, DownTrend[0] + TickSize * 2, _barColorDown);
					if(RangeType) 
					{
						DrawText("EntryText" + CurrentBar.ToString(), true, FormatPrice(lowMarker), 0, DownTrend[0] + TickSize * 2, 34, Color.FromArgb(255, ChartControl.AxisColor), new Font("Arial", 9, GraphicsUnit.Point), StringAlignment.Center, Color.FromArgb(255, ChartControl.AxisColor), Color.Empty, 0);
						DrawArrowDown("EntryArrows" + CurrentBar.ToString(), true, 0, DownTrend[0] + TickSize * 2, _barColorDown);
					}
				}
				else
				{
					RemoveDrawObject("EntryArrows" + CurrentBar.ToString());
					RemoveDrawObject("EntryText" + CurrentBar.ToString());
				}*/
				if( (Trend[0] && !Trend[1]) && CCI(_cCIPeriods)[0]> 0)
				{
					
					if(RangeType) 
					{
						DrawText("EntryText" + CurrentBar.ToString(), true, FormatPrice(highMarker), 0, UpTrend[0] - TickSize * 2, -30, Color.FromArgb(255, ChartControl.AxisColor), new Font("Arial", 9, GraphicsUnit.Point), StringAlignment.Center, Color.FromArgb(255, ChartControl.AxisColor), Color.Empty, 0);
						DrawArrowUp("EntryArrows" + CurrentBar.ToString(), true, 0, UpTrend[0] - TickSize * 2, _barColorUp);
					}
				}
				else
				if( (!Trend[0] && Trend[1]) && CCI(_cCIPeriods)[0]< 0)
				{
				if(RangeType) 
					{
						DrawText("EntryText" + CurrentBar.ToString(), true, FormatPrice(lowMarker), 0, DownTrend[0] + TickSize * 2, 34, Color.FromArgb(255, ChartControl.AxisColor), new Font("Arial", 9, GraphicsUnit.Point), StringAlignment.Center, Color.FromArgb(255, ChartControl.AxisColor), Color.Empty, 0);
						DrawArrowDown("EntryArrows" + CurrentBar.ToString(), true, 0, DownTrend[0] + TickSize * 2, _barColorDown);
					}
				}
				else
				{
					RemoveDrawObject("EntryArrows" + CurrentBar.ToString());
					RemoveDrawObject("EntryText" + CurrentBar.ToString());
				}
			}

            if (!_colorBars) return;
			
			if(!_cciOverrides && TrendUpDn || (_cciOverrides && TrendUpDn && CCIUpDn[0]))//Trend up color 	
				{
					if(ChartControl.ChartStyleType == ChartStyleType.CandleStick && showTrendCandles)//trend candles
					{
						CandleOutlineColor = _barColorUp; 
						if(Open[0]<Close[0])
							BarColor  = Color.Transparent; 
							else
							BarColor  = _barColorUp;
					}
					else if(ChartControl.ChartStyleType == ChartStyleType.CandleStick && !showTrendCandles)//solid candles
					{
						CandleOutlineColor = _barColorUp;	
						BarColor  = _barColorUp; 
					}
					else if(ChartControl.ChartStyleType != ChartStyleType.CandleStick)//bars
					{
						BarColor  = _barColorUp;
					}
				}
					
				else if(!_cciOverrides && !TrendUpDn || (_cciOverrides && !TrendUpDn && !CCIUpDn[0]))//Trend down color	
				{
					if(ChartControl.ChartStyleType == ChartStyleType.CandleStick && showTrendCandles)//trend candles
					{
						CandleOutlineColor = _barColorDown; 
						if(Open[0]<Close[0])
							BarColor  = Color.Transparent; 
							else
							BarColor  = _barColorDown;
					}
					else if(ChartControl.ChartStyleType == ChartStyleType.CandleStick && !showTrendCandles)//solid candles
					{
						CandleOutlineColor = _barColorDown;
						BarColor  = _barColorDown;
					}
					else if(ChartControl.ChartStyleType != ChartStyleType.CandleStick)//bars
					{
						BarColor  = _barColorDown;
					}
				}
					
				else if(_cciOverrides && (TrendUpDn && !CCIUpDn[0] || !TrendUpDn && CCIUpDn[0]))//Trendless color	
				{
					if(ChartControl.ChartStyleType == ChartStyleType.CandleStick && showTrendCandles)//trend candles
					{
						CandleOutlineColor = _barColorDisagreement; 
						if(Open[0]<Close[0])
							BarColor  = Color.Transparent; 
							else
							BarColor  = _barColorDisagreement;
					}
					else if(ChartControl.ChartStyleType == ChartStyleType.CandleStick && !showTrendCandles)//solid candles
					{
						CandleOutlineColor = _barColorDisagreement;
						BarColor  = _barColorDisagreement;
					}
					else if(ChartControl.ChartStyleType != ChartStyleType.CandleStick)//bars
					{
						BarColor  = _barColorDisagreement;
					}
				}	
				
				//Signal bars
				if(FirstTickOfBar && CurrentBar > 3)
				{
					if(CurrentBar - 1 >= (probeLongBar + barsTillReset) && _trend[2] && Low[1] <= Instrument.MasterInstrument.Round2TickSize(UpTrend[2]) + (ticksFromTrendline * TickSize) && Close[1] != Low[1] && Close[1] > UpTrend[2])
						probeLongBar = CurrentBar - 1;
					if(CurrentBar - 1 >= (probeShortBar + barsTillReset) && !_trend[2] && High[1] >= Instrument.MasterInstrument.Round2TickSize(DownTrend[2]) - (ticksFromTrendline * TickSize) && Close[1] != High[1] && Close[1] < DownTrend[2])
						probeShortBar = CurrentBar - 1;
				}
				
				if(CurrentBar >= probeLongBar + barsTillReset)
				{
					if(_trend[1] && Low[0] <= Instrument.MasterInstrument.Round2TickSize(UpTrend[1]) + (ticksFromTrendline * TickSize) && Close[0] != Low[0] && Close[0] > UpTrend[1])
					{
						if(ChartControl.ChartStyleType == ChartStyleType.CandleStick && showTrendCandles)//trend candles
						{
							CandleOutlineColor = _probeColorUp; 
							//if(Open[0]<Close[0])
								BarColor  = Color.Transparent; 
								//else
								//BarColor  = _probeColorUp;
						}
						else if(ChartControl.ChartStyleType == ChartStyleType.CandleStick && !showTrendCandles)//solid candles
						{
							CandleOutlineColor = _probeColorUp;	
							BarColor  = _probeColorUp; 
						}
						else if(ChartControl.ChartStyleType != ChartStyleType.CandleStick)//bars
						{
							BarColor  = _probeColorUp;
						}
					}
				}
				
				if(CurrentBar >= probeShortBar + barsTillReset)
				{
					if(!_trend[1] && High[0] >= Instrument.MasterInstrument.Round2TickSize(DownTrend[1]) - (ticksFromTrendline * TickSize) && Close[0] != High[0] && Close[0] < DownTrend[1])
					{
						if(ChartControl.ChartStyleType == ChartStyleType.CandleStick && showTrendCandles)//trend candles
						{
							CandleOutlineColor = _probeColorDown; 
							//if(Open[0]<Close[0])
								//BarColor  = Color.Transparent; 
								//else
								BarColor  = _probeColorDown;
						}
						else if(ChartControl.ChartStyleType == ChartStyleType.CandleStick && !showTrendCandles)//solid candles
						{
							CandleOutlineColor = _probeColorDown;	
							BarColor  = _probeColorDown; 
						}
						else if(ChartControl.ChartStyleType != ChartStyleType.CandleStick)//bars
						{
							BarColor  = _probeColorDown;
						}
					}
				}
		}
		
		private string FormatPrice(double iVal)
        {
            return Bars.Instrument.MasterInstrument.FormatPrice(iVal);
        }
		
        private double DTT(int nDay, double mult)
        {
            double hh = MAX(High, nDay)[0];
            double hc = MAX(Close, nDay)[0];
            double ll = MIN(Low, nDay)[0];
            double lc = MIN(Close, nDay)[0];
            return mult * Math.Max((hh - lc), (hc - ll));
        }

        #region Properties
        [Browsable(false)]	
        [XmlIgnore]		
        public DataSeries BreakTrend
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	
        [XmlIgnore]		
        public DataSeries UpTrend
        {
            get { return Values[1]; }
        }

        [Browsable(false)]
        [XmlIgnore]	
        public DataSeries DownTrend
        {
            get { return Values[2]; }
        }

        [Description("SuperTrendMode")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("01. SuperTrend Mode")]
        public SuperTrendModeSDT2 StMode
        {
            get { return _smode; }
            set { _smode = value; }
        }

        [Description("ATR/DT Period")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("02. Period")]
        public int Length
        {
            get { return _length; }
            set { _length = Math.Max(1, value); }
        }

        [Description("ATR Multiplier")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("03. Multiplier")]
        public double Multiplier
        {
            get { return _multiplier; }
            set { _multiplier = Math.Max(0.0001, value); }
        }

        [Description("Moving Average Type for smoothing")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("04. Moving Average Type")]
        public MovingAverageTypeSDT2 MaType
        {
            get { return _maType; }
            set { _maType = value; }
        }

        [Description("Smoothing Period")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("05. SmoothingPeriod (MA)")]
        public int Smooth
        {
            get { return _smooth; }
            set { _smooth = Math.Max(1, value); }
        }

		[Description("CCI Period for Momentum ")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("06. CCI Period")]
        public int CCIPeriods
        {
            get { return _cCIPeriods; }
            set { _cCIPeriods = Math.Max(1, value); }
        }
		
		[Description("CCI Overides modes")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("07. CCI Overrides")]
        public CCI_Overrides2 OverridesType
        {
            get { return _overridesType; }
            set { _overridesType = value; }
        }
		
		[Description("Number of ticks from the trend line for signal bar")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("08. Ticks from trend")]
        public int TicksFromTrendline
        {
            get { return ticksFromTrendline; }
            set { ticksFromTrendline = Math.Max(1, value); }
        }
		
		[Description("Number of bars from the last signal bar for a reset")]
        [Category("Parameters")]
        [Gui.Design.DisplayName("09. Bars for reset")]
        public int BarsTillReset
        {
            get { return barsTillReset; }
            set { barsTillReset = Math.Max(1, value); }
        }
 		
		[Description("Show Triangles when Trendline is violated?")]
        [Category("Visual")]
        [Gui.Design.DisplayName("01. Show cross markers?")]
        public bool ShowArrows
        {
            get { return _showArrows; }
            set { _showArrows = value; }
        }

        [Description("Color the bars in the direction of the trend?")]
        [Category("Visual")]
        [Gui.Design.DisplayName("02. Color Bars?")]
        public bool ColorBars
        {
            get { return _colorBars; }
            set { _colorBars = value; }
        }
		
		[Description("Show hollow trend candles or solid.")]//Added IT
		[Gui.Design.DisplayName ("03. Trend style candles?")]
        [Category("Visual")]
        public bool ShowTrendCandles
        {
            get { return showTrendCandles; }
            set { showTrendCandles = value; }
        }

        [XmlIgnore]
        [Description("Color of up bars.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("04. Up color")]
        public Color BarColorUp
        {
            get { return _barColorUp; }
            set { _barColorUp = value; }
        }

        [Browsable(false)]
        public string BarColorUpSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(_barColorUp); }
            set { _barColorUp = Gui.Design.SerializableColor.FromString(value); }
        }

        [XmlIgnore]
        [Description("Color of down bars.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("05. Down color")]
        public Color BarColorDown
        {
            get { return _barColorDown; }
            set { _barColorDown = value; }
        }

        [Browsable(false)]
        public string BarColorDownSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(_barColorDown); }
            set { _barColorDown = Gui.Design.SerializableColor.FromString(value); }
        }
		
		[XmlIgnore]
        [Description("Color of disagreement bars (if CCI and Transition selected).")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("06. Disagreement color")]
        public Color BarColorDisagreement
        {
            get { return _barColorDisagreement; }
            set { _barColorDisagreement = value; }
        }

        [Browsable(false)]
        public string BarColorDisagreementSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(_barColorDisagreement); }
            set { _barColorDisagreement = Gui.Design.SerializableColor.FromString(value); }
        }
		
		[Description("Show the trend line cross dots. Dot color is adjusted in the Plots section.")]//Added IT
		[Gui.Design.DisplayName ("07. Trend cross dots?")]
        [Category("Visual")]
        public bool ShowTrendCrossDots
        {
            get { return showTrendCrossDots; }
            set { showTrendCrossDots = value; }
        }
		
		[Description("Show the entry signal with an arrow.")]//Added IT
		[Gui.Design.DisplayName ("08. Show entries?")]
        [Category("Visual")]
        public bool ShowEntries
        {
            get { return showEntries; }
            set { showEntries = value; }
        }
		
		[XmlIgnore]
        [Description("Color of probe bar long.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("09. Probe Long color")]
        public Color ProbeColorUp
        {
            get { return _probeColorUp; }
            set { _probeColorUp = value; }
        }

        [Browsable(false)]
        public string ProbeColorUpSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(_probeColorUp); }
            set { _probeColorUp = Gui.Design.SerializableColor.FromString(value); }
        }
		
		[XmlIgnore]
        [Description("Color of probe bar short.")]
        [Category("Visual")]
        [Gui.Design.DisplayNameAttribute("10. Probe Short color")]
        public Color ProbeColorDown
        {
            get { return _probeColorDown; }
            set { _probeColorDown = value; }
        }

        [Browsable(false)]
        public string ProbeColorDownSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(_probeColorDown); }
            set { _probeColorDown = Gui.Design.SerializableColor.FromString(value); }
        }
		
        [Browsable(false)]
        public BoolSeries Trend
        {
            get { return _trend; }
        }

        #endregion
    }
}
//namespace SuperDuperTrend2.Utility
//{

public enum SuperTrendModeSDT2
{
    ATR,
    DualThrust,
    Adaptive
}

public enum CCI_Overrides2
{
	None,
	Override,
}

public enum MovingAverageTypeSDT2
{
    SMA,
    SMMA,
    TMA,
    WMA,
    VWMA,
    TEMA,
    HMA,
    EMA,
    VMA
}

//}


#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SuperDuperTrend2[] cacheSuperDuperTrend2 = null;

        private static SuperDuperTrend2 checkSuperDuperTrend2 = new SuperDuperTrend2();

        /// <summary>
        /// SuperDuperTrend2 Indicator modified from TSSuperTrend (TradingStudies.com (Version 2.2)) by Integrity-Traders.com
        /// </summary>
        /// <returns></returns>
        public SuperDuperTrend2 SuperDuperTrend2(int barsTillReset, int cCIPeriods, int length, MovingAverageTypeSDT2 maType, double multiplier, CCI_Overrides2 overridesType, int smooth, SuperTrendModeSDT2 stMode, int ticksFromTrendline)
        {
            return SuperDuperTrend2(Input, barsTillReset, cCIPeriods, length, maType, multiplier, overridesType, smooth, stMode, ticksFromTrendline);
        }

        /// <summary>
        /// SuperDuperTrend2 Indicator modified from TSSuperTrend (TradingStudies.com (Version 2.2)) by Integrity-Traders.com
        /// </summary>
        /// <returns></returns>
        public SuperDuperTrend2 SuperDuperTrend2(Data.IDataSeries input, int barsTillReset, int cCIPeriods, int length, MovingAverageTypeSDT2 maType, double multiplier, CCI_Overrides2 overridesType, int smooth, SuperTrendModeSDT2 stMode, int ticksFromTrendline)
        {
            if (cacheSuperDuperTrend2 != null)
                for (int idx = 0; idx < cacheSuperDuperTrend2.Length; idx++)
                    if (cacheSuperDuperTrend2[idx].BarsTillReset == barsTillReset && cacheSuperDuperTrend2[idx].CCIPeriods == cCIPeriods && cacheSuperDuperTrend2[idx].Length == length && cacheSuperDuperTrend2[idx].MaType == maType && Math.Abs(cacheSuperDuperTrend2[idx].Multiplier - multiplier) <= double.Epsilon && cacheSuperDuperTrend2[idx].OverridesType == overridesType && cacheSuperDuperTrend2[idx].Smooth == smooth && cacheSuperDuperTrend2[idx].StMode == stMode && cacheSuperDuperTrend2[idx].TicksFromTrendline == ticksFromTrendline && cacheSuperDuperTrend2[idx].EqualsInput(input))
                        return cacheSuperDuperTrend2[idx];

            lock (checkSuperDuperTrend2)
            {
                checkSuperDuperTrend2.BarsTillReset = barsTillReset;
                barsTillReset = checkSuperDuperTrend2.BarsTillReset;
                checkSuperDuperTrend2.CCIPeriods = cCIPeriods;
                cCIPeriods = checkSuperDuperTrend2.CCIPeriods;
                checkSuperDuperTrend2.Length = length;
                length = checkSuperDuperTrend2.Length;
                checkSuperDuperTrend2.MaType = maType;
                maType = checkSuperDuperTrend2.MaType;
                checkSuperDuperTrend2.Multiplier = multiplier;
                multiplier = checkSuperDuperTrend2.Multiplier;
                checkSuperDuperTrend2.OverridesType = overridesType;
                overridesType = checkSuperDuperTrend2.OverridesType;
                checkSuperDuperTrend2.Smooth = smooth;
                smooth = checkSuperDuperTrend2.Smooth;
                checkSuperDuperTrend2.StMode = stMode;
                stMode = checkSuperDuperTrend2.StMode;
                checkSuperDuperTrend2.TicksFromTrendline = ticksFromTrendline;
                ticksFromTrendline = checkSuperDuperTrend2.TicksFromTrendline;

                if (cacheSuperDuperTrend2 != null)
                    for (int idx = 0; idx < cacheSuperDuperTrend2.Length; idx++)
                        if (cacheSuperDuperTrend2[idx].BarsTillReset == barsTillReset && cacheSuperDuperTrend2[idx].CCIPeriods == cCIPeriods && cacheSuperDuperTrend2[idx].Length == length && cacheSuperDuperTrend2[idx].MaType == maType && Math.Abs(cacheSuperDuperTrend2[idx].Multiplier - multiplier) <= double.Epsilon && cacheSuperDuperTrend2[idx].OverridesType == overridesType && cacheSuperDuperTrend2[idx].Smooth == smooth && cacheSuperDuperTrend2[idx].StMode == stMode && cacheSuperDuperTrend2[idx].TicksFromTrendline == ticksFromTrendline && cacheSuperDuperTrend2[idx].EqualsInput(input))
                            return cacheSuperDuperTrend2[idx];

                SuperDuperTrend2 indicator = new SuperDuperTrend2();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.BarsTillReset = barsTillReset;
                indicator.CCIPeriods = cCIPeriods;
                indicator.Length = length;
                indicator.MaType = maType;
                indicator.Multiplier = multiplier;
                indicator.OverridesType = overridesType;
                indicator.Smooth = smooth;
                indicator.StMode = stMode;
                indicator.TicksFromTrendline = ticksFromTrendline;
                Indicators.Add(indicator);
                indicator.SetUp();

                SuperDuperTrend2[] tmp = new SuperDuperTrend2[cacheSuperDuperTrend2 == null ? 1 : cacheSuperDuperTrend2.Length + 1];
                if (cacheSuperDuperTrend2 != null)
                    cacheSuperDuperTrend2.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSuperDuperTrend2 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// SuperDuperTrend2 Indicator modified from TSSuperTrend (TradingStudies.com (Version 2.2)) by Integrity-Traders.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SuperDuperTrend2 SuperDuperTrend2(int barsTillReset, int cCIPeriods, int length, MovingAverageTypeSDT2 maType, double multiplier, CCI_Overrides2 overridesType, int smooth, SuperTrendModeSDT2 stMode, int ticksFromTrendline)
        {
            return _indicator.SuperDuperTrend2(Input, barsTillReset, cCIPeriods, length, maType, multiplier, overridesType, smooth, stMode, ticksFromTrendline);
        }

        /// <summary>
        /// SuperDuperTrend2 Indicator modified from TSSuperTrend (TradingStudies.com (Version 2.2)) by Integrity-Traders.com
        /// </summary>
        /// <returns></returns>
        public Indicator.SuperDuperTrend2 SuperDuperTrend2(Data.IDataSeries input, int barsTillReset, int cCIPeriods, int length, MovingAverageTypeSDT2 maType, double multiplier, CCI_Overrides2 overridesType, int smooth, SuperTrendModeSDT2 stMode, int ticksFromTrendline)
        {
            return _indicator.SuperDuperTrend2(input, barsTillReset, cCIPeriods, length, maType, multiplier, overridesType, smooth, stMode, ticksFromTrendline);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// SuperDuperTrend2 Indicator modified from TSSuperTrend (TradingStudies.com (Version 2.2)) by Integrity-Traders.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SuperDuperTrend2 SuperDuperTrend2(int barsTillReset, int cCIPeriods, int length, MovingAverageTypeSDT2 maType, double multiplier, CCI_Overrides2 overridesType, int smooth, SuperTrendModeSDT2 stMode, int ticksFromTrendline)
        {
            return _indicator.SuperDuperTrend2(Input, barsTillReset, cCIPeriods, length, maType, multiplier, overridesType, smooth, stMode, ticksFromTrendline);
        }

        /// <summary>
        /// SuperDuperTrend2 Indicator modified from TSSuperTrend (TradingStudies.com (Version 2.2)) by Integrity-Traders.com
        /// </summary>
        /// <returns></returns>
        public Indicator.SuperDuperTrend2 SuperDuperTrend2(Data.IDataSeries input, int barsTillReset, int cCIPeriods, int length, MovingAverageTypeSDT2 maType, double multiplier, CCI_Overrides2 overridesType, int smooth, SuperTrendModeSDT2 stMode, int ticksFromTrendline)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SuperDuperTrend2(input, barsTillReset, cCIPeriods, length, maType, multiplier, overridesType, smooth, stMode, ticksFromTrendline);
        }
    }
}
#endregion
